/*
 * AbstractBuilder.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import org.jabberstudio.jso.io.StreamBuilder;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public abstract class AbstractBuilder implements StreamBuilder {
    //Fields
    private Status  _Current = StreamBuilder.READY;
    
    //Constructors
    /**
     * <p>
     * Creates a new <code>AbstractBuilder</code> using fields initialized
     * to default values.</p>
     *
     */
    public AbstractBuilder() {
    }
    
    //Methods
    /** <p>
     * Retrieves the current status of this <code>AbstractBuilder</code>.  The
     * status is ued to determine when certain methods (such as <code>{@link
     * #getNode}</code>, <code>{@link #buildExtendedData}</code>, and
     * <code>{@link #addExtendedData}</code>) can successfully be called.</p>
     *
     * @return  The current status for this <code>AbstractBuilder</code>.
     */
    public Status getStatus() {
        return _Current;
    }
    /**
     * <p>
     * Sets the current status of this <code>AbstractBuilder</code>.</p>
     *
     * <p>
     * The value of <code>status</code> cannot be <code>null</code>, or an
     * <Code>IllegalArgumentException</code> thrown.</p>
     *
     * @param status The new status
     * @throws IllegalArgumentException If the parameter is invalid.
     */
    protected void setStatus(Status status) throws IllegalArgumentException {
        _Current = status;
    }
}