/*
 * InfoQueryPacket.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;



import org.jabberstudio.jso.InfoQuery;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class InfoQueryPacket extends BasicPacketNode implements InfoQuery {
    //"Constants"
    public static final NSI NAME = new NSI("iq", null);

    //Constructors
    /** Creates a new instance of InfoQueryPacket */
    public InfoQueryPacket(StreamDataFactory sdf) {
        this (sdf, NAME);
    }
    public InfoQueryPacket(StreamDataFactory sdf, NSI name) {
        super(sdf, name);
    }
    private InfoQueryPacket(StreamElement parent, InfoQueryPacket base) {
        super(parent, base);
    }
    
    //Methods
    protected Packet.Type determineType(String name) {
        Packet.Type     type;
        
        if          (Utilities.equateStrings(GET.toString(), name)) {
            type = GET;
        } else if   (Utilities.equateStrings(SET.toString(), name)) {
            type = SET;
        } else if   (Utilities.equateStrings(RESULT.toString(), name)) {
            type = RESULT;
        } else {
            type = super.determineType(name);
        }
        
        return type;
    }
    
    
    public void checkNSI(NSI check) throws IllegalArgumentException {
        if (NSI.NS_LOOSE_COMPARATOR.compare(check, NAME) != 0)
            throw new IllegalArgumentException("Name must be 'iq'");
    }
    
    /** <p>
     * Creates an "almost-exact" duplicate of this <code>StreamNode</code>,
     * using the given parent <code>StreamElement</code>.  By "almost-exact",
     * all information for this <code>StreamNode</code>, and its contained
     * objects (obtained via <code>{@link #listNodes}</code>) are duplicated.
     * The parent for this <code>StreamNode</code> is one specified by
     * <code>p</code>, rather than that referenced by this
     * <code>StreamNode</code>.</p>
     *
     * <p>
     * The value of <code>p</code> may be <code>null</code>, in which case no
     * parent is attached to the newly created <code>StreamNode</code>.</p>
     *
     * @return the "almost-exact" duplicate of this <code>StreamNode</code>.
     */
    public StreamObject copy(StreamElement p) {
        return new InfoQueryPacket(p, this);
    }
}