/*
 * MessagePacket.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Iterator;
import java.util.Locale;

import org.jabberstudio.jso.Message;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;


/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class MessagePacket extends BasicPacketNode implements Message {
    //"Constants"
    public static final NSI NAME = new NSI("message", null);
    
    public static final NSI NAME_BODY = new NSI("body", null);
    public static final NSI NAME_SUBJECT = new NSI("subject", null);
    public static final NSI NAME_THREAD = new NSI("thread", null);

    //Constructors
    public MessagePacket(StreamDataFactory sdf) {
        this(sdf, NAME);
    }
    public MessagePacket(StreamDataFactory sdf, NSI name) {
        super(sdf, name);
    }
    public MessagePacket(StreamElement parent, MessagePacket base) {
        super(parent, base);
    }
    
    //Methods
    public void checkNSI(NSI check) throws IllegalArgumentException {
        if (NSI.NS_LOOSE_COMPARATOR.compare(check, NAME) != 0)
            throw new IllegalArgumentException("Name must be 'message'");
    }
    
    public StreamObject copy(StreamElement p) {
        return new MessagePacket(p, this);
    }
    
    public void setType(Packet.Type type) throws IllegalArgumentException {
        if ((type == null) || (type == ERROR) || (type instanceof Message.Type))
            super.setType(type);
        else
            throw new IllegalArgumentException("Only ERROR or message types permitted");
    }
    
    protected Packet.Type determineType(String name) {
        Packet.Type     type;
        
        if          (Utilities.equateStrings(CHAT.toString(), name)) {
            type = CHAT;
        } else if   (Utilities.equateStrings(GROUPCHAT.toString(), name)) {
            type = GROUPCHAT;
        } else if   (Utilities.equateStrings(HEADLINE.toString(), name)) {
            type = HEADLINE;
        } else if   (Utilities.equateStrings(NORMAL.toString(), name)) {
            type = NORMAL;
        } else {
            type = super.determineType(name);
        }
        
        return type;
    }

    public boolean hasBody() {
        return hasBody(null);
    }
    public boolean hasBody(Locale l) {
        return !findElementsForLocale(NAME_BODY.getLocalName(), getNamespaceURI(), l).isEmpty();
    }
    public String getBody() {
        return getBody(null);
    }
    public String getBody(Locale l) {
        Iterator        itr = findElementsForLocale(NAME_BODY.getLocalName(), getNamespaceURI(), l).iterator();
        String          body = "";
        
        if (itr.hasNext()) {
            body = ((StreamElement)itr.next()).normalizeText();
        }
        
        return body;
    }
    public void setBody(String b) {
        setBody(null, b);
    }
    public void setBody(Locale l, String b) {
        Iterator        itr = findElementsForLocale(NAME_BODY.getLocalName(), getNamespaceURI(), l).iterator();
        StreamElement   elem;
        
        while (itr.hasNext())
            ((StreamElement)itr.next()).detach();
        
        if (b != null) {
            elem = addElement(NAME_BODY.getLocalName());
            elem.addText(b);
            if (l != null)
                elem.setDeclaredLocale(l);
        }
    }
    
    public String getThread() {
        StreamElement   elem = getFirstElement(NAME_THREAD.getLocalName(), getNamespaceURI());
        String          thread = "";

        if (elem != null) {
            thread = elem.normalizeTrimText();
        }

        return thread;
    }
    public void setThread(String t) {
        clearElements(NAME_THREAD.getLocalName(), getNamespaceURI());
        
        if (t != null) {
            addElement(NAME_THREAD.getLocalName()).addText(t);
        }
    }

    public boolean hasSubject() {
        return hasSubject(null);
    }
    public boolean hasSubject(Locale l) {
        return !findElementsForLocale(NAME_SUBJECT.getLocalName(), getNamespaceURI(), l).isEmpty();
    }
    public String getSubject() {
        return getSubject(null);
    }
    public String getSubject(Locale l) {
        Iterator    itr = findElementsForLocale(NAME_SUBJECT.getLocalName(), getNamespaceURI(), l).iterator();
        String      subject = "";
        
        if (itr.hasNext())
            subject = ((StreamElement)itr.next()).normalizeTrimText();
        
        return subject;
    }
    public void setSubject(String s) {
        setSubject(null, s);
    }
    public void setSubject(Locale l, String s) {
        Iterator        itr = findElementsForLocale(NAME_SUBJECT.getLocalName(), getNamespaceURI(), l).iterator();
        StreamElement   elem;
        
        while (itr.hasNext())
            ((StreamElement)itr.next()).detach();
        
        if (s != null) {
            elem = addElement(NAME_SUBJECT.getLocalName());
            elem.addText(s);
            if (l != null)
                elem.setDeclaredLocale(l);
        }
    }
}