/*
 * StreamErrorNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jaxen.JaxenException;
import org.jaxen.XPath;

import org.saxpath.SAXPathException;

import org.jabberstudio.jso.ErrorCondition;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.PacketError;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamError;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.io.StreamParseState;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 */
public class StreamErrorNode extends AbstractPacket implements StreamError {
    //"Constants"
    public static final NSI     ELEMNAME_TEXT   = new NSI("text", CONDITION_NAMESPACE);
    
    public static final NSI     NAME            = new NSI("error", Utilities.STREAM_NAMESPACE);

    public static final Map     DEFINED_CONDITIONS;
    
    static {
        Map     temp = new java.util.TreeMap();
        NSI     name;
        
        name = new NSI(BAD_FORMAT_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(BAD_NAMESPACE_PREFIX_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(CONFLICT_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(CONNECTION_TIMEOUT_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(HOST_GONE_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(HOST_UNKNOWN_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(IMPROPER_ADDRESSING_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(INTERNAL_SERVER_ERROR_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(INVALID_FROM_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(INVALID_ID_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(INVALID_XML_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(NOT_AUTHORIZED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(POLICY_VIOLATION_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(REMOTE_CONNECTION_FAILED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(RESOURCE_CONSTRAINT_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(RESTRICTED_XML_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(SEE_OTHER_HOST_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(SYSTEM_SHUTDOWN_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(UNDEFINED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(UNSUPPORTED_ENCODING_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(UNSUPPORTED_STANZA_TYPE_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(UNSUPPORTED_VERSION_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        name = new NSI(XML_NOT_WELL_FORMED_CONDITION, CONDITION_NAMESPACE);
        temp.put(name.getLocalName(), name);
        
        DEFINED_CONDITIONS = java.util.Collections.unmodifiableMap(temp);
    }
    
    //Fields
    private XPath   _AppCondPath;

    //Constructors
    public StreamErrorNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected StreamErrorNode(StreamElement parent, StreamErrorNode base) {
        super(parent, base);
    }
    
    //Methods
    protected XPath getApplicationConditionXPath() {
        if (_AppCondPath == null) {
        if (_AppCondPath == null) {
            try {
                _AppCondPath = getDataFactory().createXPath("*[namespace-uri()!='" + getNamespaceURI() + "' and namespace-uri()!='" + CONDITION_NAMESPACE + "']");
            } catch (SAXPathException saxpe) {
            }
        }
        
         return _AppCondPath;
        }
        
        return _AppCondPath;
    }
    
    public StreamBuilder createBuilder() {
        return new AbstractPacketBuilder(this) {
            protected StreamBuilder.Status processNestedElementStart(StreamParseState s) {
                StreamElement       ctx = s.getContext();
                NSI.Comparator      cmpr = NSI.NS_LOOSE_COMPARATOR;
                NSI                 name = s.getName();
                String              uri = name.getNamespaceURI();
                String              ln = name.getLocalName();
                StreamDataFactory   sdf = ctx.getDataFactory();
                StreamBuilder       b;


                if (Utilities.equateStrings(StreamError.CONDITION_NAMESPACE, uri) && Utilities.equateStrings("text", ln)) {
                    setExtendedData(sdf.createElementBuilder(name));
                } else {
                    //Setup "extended data"
                    setExtendedData(sdf.createElementBuilder(name, ErrorCondition.class));
                }

                return EXTENDED_DATA;
            }
            
            public StreamNode getNode() throws IllegalStateException {
                StreamError     err = (StreamError)super.getNode();
                String          txt = err.normalizeText();
                String          cond = err.getDefinedCondition();
                
                
                if (!Utilities.isValidString(cond))
                    err.setDefinedCondition(UNDEFINED_CONDITION);
                if (Utilities.isValidString(txt)) {
                    err.clearText();
                    err.setText(txt);
                }
                
                return err;
            }
        };
    }
    
    public PacketError getError() {
        return null;
    }
    public void setError(PacketError err) throws IllegalArgumentException {
    }
    
    public String getText() {
        StreamElement   text = getFirstElement("text", CONDITION_NAMESPACE);
        
        return ((text != null) ? text.normalizeTrimText() : "");
    }
    public void setText(String txt) {
        Iterator    itr = listElements(ELEMNAME_TEXT).iterator();
        
        while (itr.hasNext())
            ((StreamElement)itr.next()).detach();
        
        if (txt != null)
            addElement(ELEMNAME_TEXT).addText(txt);
    }
    
    public List listConditions() {
        return listElements(ErrorCondition.class);
    }
    
    public NSI getApplicationCondition() {
        StreamElement   elem;
        NSI             name = null;
        
        try {
            elem = (StreamElement)getApplicationConditionXPath().selectSingleNode(this);
            
            if (elem != null)
                name = elem.getNSI();
        } catch (JaxenException je) {
        }
        
        return name;
    }
    public void setApplicationCondition(NSI name) {
        Iterator    itr;
        
        //Validate input
        if (name != null) {
            if (!Utilities.isValidString(name.getLocalName()))
                throw new IllegalArgumentException("localname cannot be null or \"\"");
            if (Utilities.equateStrings(name.getNamespaceURI(), getNSI().getNamespaceURI()))
                throw new IllegalArgumentException("namespace URI cannot be in stanza namespace");
            if (Utilities.equateStrings(name.getNamespaceURI(), CONDITION_NAMESPACE))
                throw new IllegalArgumentException("namespace URI cannot be in stanza-conditions namespace");
        }
        
        //Out the old...
        try {
            itr = getApplicationConditionXPath().selectNodes(this).iterator();
            
            while (itr.hasNext())
                ((StreamElement)itr.next()).detach();
        } catch (JaxenException je) {
        }
        
        //...And in with the new!
        if (name != null) {
            addElement(name, ErrorCondition.class);
        }
    }
    
    public String getDefinedCondition() {
        StreamElement   condition = getFirstElement(null, CONDITION_NAMESPACE, ErrorCondition.class);
        String          name = "";
        
        if (condition != null)
            name = condition.getLocalName();
        
        return name;
    }
    public void setDefinedCondition(String name) throws IllegalArgumentException {
        Iterator        itr = listElements(null, CONDITION_NAMESPACE, ErrorCondition.class).iterator();
        NSI             nsi;
        
        //Remove any existing
        while (itr.hasNext()) {
            ((StreamElement)itr.next()).detach();
        }

        if (Utilities.isValidString(name)) {
            if ((nsi = (NSI)DEFINED_CONDITIONS.get(name)) == null)
                throw new IllegalArgumentException("\"" + nsi + "\" is not a defined condition");
            addElement(nsi, ErrorCondition.class);
        }
    }
    
    public StreamObject copy(StreamElement parent) {
        return new StreamErrorNode(parent, this);
    }
}