/*
 * Parser.java
 *
 * Created on August 22, 2004, 1:25 PM
 */

package net.outer_planes.jso.ixp;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.jclark.xml.tok.ContentToken;
import com.jclark.xml.tok.Encoding;
import com.jclark.xml.tok.EmptyTokenException;
import com.jclark.xml.tok.ExtensibleTokenException;
import com.jclark.xml.tok.PartialTokenException;
import com.jclark.xml.tok.StringConversionCache;
import com.jclark.xml.tok.Token;
import com.jclark.xml.tok.TokenException;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.io.StreamParseState;
import org.jabberstudio.jso.io.StreamSource;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class IxpParser {
    //Fields

    private Buffer                  _Buffer;
    private Context                 _Ctx;

    //Constructors
    public IxpParser(StreamElement ctx, StreamSource src) {
        _Buffer = new Buffer(src);
        _Ctx = new Context(ctx);
    }
    
    //Methods
    private Encoding findEncoding() {
        Context     ctx = getContext();
        Buffer      buffer = getBuffer();
        byte[]      data = buffer.getData();
        int         len = buffer.getLength();
        Token       token = new Token();
        Encoding    enc = Encoding.getInitialEncoding(data, 0, len, token);
        
        buffer.eat(token.getTokenEnd());
        ctx.configureEncoding(enc);
        
        return enc;
    }
    
    private Buffer getBuffer() {
        return _Buffer;
    }
    
    private Context getContext() {
        return _Ctx;
    }
    
    public boolean isEOD() {
        return getContext().isEOD();
    }
    
    public StreamParseState process() throws IOException, IxpException {
        Context         ctx = getContext();
        Buffer          buffer = getBuffer();
        ContentToken    ct = new ContentToken();
        byte[]          data;
        int             length, type;
        boolean         cont = true, more = false;
        
        while (!ctx.hasStates() && cont) {
            data = buffer.getData();
            length = buffer.getLength();
            more = (more ? true : (length == 0));
            
            if (more) {
                //Check for more data
                buffer.fill();
                data = buffer.getData();
                length = buffer.getLength();
            }
            
            try {
                type = tokenize(ct);
                length = ct.getTokenEnd();
                more = (length == 0);
                
                switch(type) {
                    case Encoding.TOK_XML_DECL:
                        break;
                    case Encoding.TOK_START_TAG_WITH_ATTS:
                    case Encoding.TOK_START_TAG_NO_ATTS:
                        handleElementStart(ct, data, false);
                        break;
                    case Encoding.TOK_END_TAG:
                        handleElementEnd(ct, data, false);
                        break;
                    case Encoding.TOK_EMPTY_ELEMENT_WITH_ATTS:
                    case Encoding.TOK_EMPTY_ELEMENT_NO_ATTS:
                        handleElementStart(ct, data, true);
                        handleElementEnd(ct, data, true);
                        break;
                    case Encoding.TOK_CDATA_SECT_OPEN:
                        ctx.startCDATA();
                        break;
                    case Encoding.TOK_CDATA_SECT_CLOSE:
                        ctx.endCDATA();
                        break;
                    case Encoding.TOK_DATA_NEWLINE:
                    case Encoding.TOK_DATA_CHARS:
                    case Encoding.TOK_ENTITY_REF:
                        ctx.characters(ctx.getStringConverter().convert(data, 0, length, false));
                        break;
                    case Encoding.TOK_MAGIC_ENTITY_REF:
                    case Encoding.TOK_CHAR_REF:
                        ctx.characters(Character.toString(ct.getRefChar()));
                        break;
                    default:
                        //Sorry, we're only a subset of XML...
                        break;
                }
                buffer.eat(length);
            } catch (PartialTokenException pte) {
                checkBuffer(pte);
                
                if (more) {
                    cont = false;
                } else {
                    cont = true;
                    more = true;
                }
            } catch (ExtensibleTokenException xte) {
                checkBuffer(xte);
                
                if (more) {
                    cont = false;
                } else {
                    cont = true;
                    more = true;
                }
            } catch (EmptyTokenException ete) {
                checkBuffer(ete);
                
                if (more) {
                    cont = false;
                } else {
                    cont = true;
                    more = true;
                }
            } catch (TokenException te) {
                IxpException    ixp = new InvalidXMLException(te.getMessage());

                ixp.initCause(te);
                throw ixp;
            }
        }
        
        return ctx.popState();
    }
    private int tokenize(ContentToken ct) throws TokenException {
        Context         ctx = getContext();
        Buffer          buffer = getBuffer();
        Encoding        enc = ctx.getEncoding();
        byte[]          data = buffer.getData();
        int             len = buffer.getLength();
        int             type;
        
        if (enc == null) {
            enc = findEncoding();
            data = buffer.getData();
            len = buffer.getLength();
        }
        
        if (ctx.isInCData()) {
            type = enc.tokenizeCdataSection(data, 0, len, ct);
        } else {
            type = enc.tokenizeContent(data, 0, len, ct);
        }

        return type;
    }
    
    private void handleElementStart(ContentToken ct, byte[] data, boolean empty) throws IxpException {
        Context                 ctx = getContext();
        Encoding                enc = ctx.getEncoding();
        StringConversionCache   scc = ctx.getStringConverter();
        NamespaceManager        nsm;
        Map                     props = new java.util.HashMap();
        Map.Entry               entry;
        Iterator                itr;
        Map                     attrs = new java.util.TreeMap(NSI.STRICT_COMPARATOR), nss = new java.util.HashMap();
        String                  qname, lname, prefix, uri, value;
        int                     start = enc.getMinBytesPerChar(), end = ct.getNameEnd();

        //Setup "attributes"
        buildAttributes(ct, data, props);
        
        //Separate out namespace declarations
        itr = props.entrySet().iterator();
        while (itr.hasNext()) {
            //Separate attrributes from namespaces
            entry = (Map.Entry)itr.next();
            qname = (String)entry.getKey();
            value = (String)entry.getValue();
            
            if (qname.startsWith("xmlns")) {
                prefix = findLocalName(qname);
                
                if (prefix.equals("xmlns"))
                    prefix = "";
                nss.put(prefix, value);
                
                itr.remove();
            }
        }
        nsm = ctx.pushNamespaces(nss);

        //Setup attributes
        itr = props.entrySet().iterator();
        while (itr.hasNext()) {
            entry = (Map.Entry)itr.next();
            value = (String)entry.getValue();
            qname = (String)entry.getKey();
            prefix = findPrefix(qname);
            lname = findLocalName(qname);
            uri = (Utilities.isValidString(prefix) ? nsm.getNamespaceURI(prefix) : null);
            
            attrs.put(new NSI(lname, uri), value);
        }
        
        qname = scc.convert(data, start, end, false);
        prefix = findPrefix(qname);
        lname = findLocalName(qname);
        uri = nsm.getNamespaceURI(prefix);
        
        ctx.startElement(new NSI(lname, uri), nss, attrs);
    }
    private void handleElementEnd(ContentToken ct, byte[] data, boolean empty) throws IxpException {
        Context                 ctx = getContext();
        Encoding                enc = ctx.getEncoding();
        StringConversionCache   scc = ctx.getStringConverter();
        NamespaceManager        nsm = ctx.getNamespaces();
        String                  qname, prefix, uri, lname;
        int                     start, end;
        
        start = (empty ? enc.getMinBytesPerChar() : (enc.getMinBytesPerChar() * 2));
        end = ct.getNameEnd();
        qname = scc.convert(data, start, end, false);
        prefix = findPrefix(qname);
        lname = findLocalName(qname);
        uri = nsm.getNamespaceURI(prefix);
        
        ctx.endElement(new NSI(lname, uri));
    }
    
    private String findPrefix(String qname) {
        String  prefix;
        int     pos = qname.indexOf(':');
        
        if (pos != -1) {
            prefix = qname.substring(0, pos);
        } else {
            prefix = "";
        }
        
        return prefix;
    }
    private String findLocalName(String qname) {
        String  ln;
        int     pos = qname.indexOf(':');
        
        if (pos != -1) {
            ln = qname.substring(pos + 1);
        } else {
            ln = qname;
        }
        
        return ln;
    }
    
    private void buildAttributes(ContentToken ct, byte[] data, Map attrs) throws IxpException {
        StringConversionCache   scc = getContext().getStringConverter();
        String                  name, value;
        int                     start, end, idx = ct.getAttributeSpecifiedCount();

        while ((idx--) > 0) {
            start = ct.getAttributeNameStart(idx);
            end = ct.getAttributeNameEnd(idx);
            name = scc.convert(data, start, end, false);
            
            start = ct.getAttributeValueStart(idx);
            end = ct.getAttributeValueEnd(idx);
            value = formatAttributeValue(data, start, end);
            
            attrs.put(name, value);
        }
    }
    private String formatAttributeValue(byte[] data, int start, int end) throws IxpException {
        Encoding                enc = getContext().getEncoding();
        StringConversionCache   scc = getContext().getStringConverter();
        Token                   token = new Token();
        StringBuffer            sb = new StringBuffer();
        int                     next, type;
        
        while (start < end) {
            try {
                type = enc.tokenizeAttributeValue(data, start, end, token);
                next = token.getTokenEnd();
            } catch (ExtensibleTokenException ete) {
                type = ete.getTokenType();
                next = token.getTokenEnd();
            } catch (TokenException te) {
                IxpException    ixp = new InvalidXMLException(te.getMessage());
                
                ixp.initCause(te);
                throw ixp;
            }
            
            switch(type) {
                case Encoding.TOK_DATA_CHARS:
                case Encoding.TOK_DATA_NEWLINE:
                case Encoding.TOK_ATTRIBUTE_VALUE_S:
                    sb.append(scc.convert(data, start, next, false));
                    break;
                case Encoding.TOK_MAGIC_ENTITY_REF:
                case Encoding.TOK_CHAR_REF:
                    sb.append(token.getRefChar());
                    break;
            }
            start = next;
        }
        
        return sb.toString();
    }
    
    private void checkBuffer(Exception e) throws IxpException {
        Buffer      buffer = getBuffer();
        Context     ctx = getContext();
        
        if (buffer.isEOF()) {
            if (ctx.isEOD()) {
                IxpException    ixp = new InvalidXMLException(e.getMessage());

                ixp.initCause(e);
                throw ixp;
            } else {
                ctx.setEOD(true);
            }
        }
    }
}