/*
 * AuthQueryNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.core;


import java.util.Iterator;
import java.util.Set;

import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.core.AuthQuery;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.4
 */
public class AuthQueryNode extends ExtensionNode implements AuthQuery {
    //Constructors
    public AuthQueryNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected AuthQueryNode(StreamElement parent, AuthQueryNode aqn) {
        super(parent, aqn);
    }
    
    //Methods
    public String getUsername() {
        return getField("username");
    }
    public void setUsername(String usr) {
        setField("username", usr);
    }
    public void clearUsername() {
        unsetField("username");
    }
    
    public String getResource() {
        return getField("resource");
    }
    public void setResource(String res) {
        setField("resource", res);
    }
    public void clearResource() {
        unsetField("resource");
    }
    
    public String getPassword() {
        return getField("password");
    }
    public void setPassword(String pass) {
        setField("password", pass);
    }
    public void clearPassword() {
        unsetField("password");
    }
    
    public String getDigest() {
        return getField("digest");
    }
    public void setDigest(String digest) {
        setField("digest", digest);
    }
    public void clearDigest() {
        unsetField("digest");
    }
    
    public Set getFieldNames() {
        Set         fields = new java.util.TreeSet();
        Iterator    itr = listElements(getDataFactory().createNSI(null, getNamespaceURI())).iterator();
        
        while (itr.hasNext()) {
            StreamElement   field = (StreamElement)itr.next();
            String          ln = field.getLocalName();
            
            fields.add(field.getLocalName());
        }
        
        return fields;
    }
    public String getField(String name) throws IllegalArgumentException {
        StreamElement   elem;
        String          value = null;
        
        //Validate name
        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("Name cannot be null or \"\"");
        
        //Retrieve and "normalize"
        elem = getFirstElement(name, getNamespaceURI());
        if (elem != null)
            value = elem.normalizeTrimText();
        
        return value;
    }
    public void setField(String name, String value) throws IllegalArgumentException {
        //Removed existing field(s) -- Validates name as a side-effect
        clearField(name);
        
        //Add the field
        addElement(name).addText(value);
    }
    public void clearField(String name) throws IllegalArgumentException {
        Iterator        itr;

        //Validate name
        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("Name cannot be null or \"\"");
        
        //Remove existing field(s)
        itr = listElements(name, getNamespaceURI()).iterator();
        while (itr.hasNext())
            remove((StreamElement)itr.next());
    }
    public void unsetField(String name) throws IllegalArgumentException {
        clearField(name);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new AuthQueryNode(parent, this);
    }
}