/*
 * FTProfileNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.sift;


import java.util.Date;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.format.DateTimeProfileFormat;
import org.jabberstudio.jso.util.ByteCodec;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.sift.FileTransferProfile;

import net.outer_planes.jso.ElementNode;
import net.outer_planes.jso.util.DateTimeProfileWrapper;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class FTProfileNode extends ElementNode implements FileTransferProfile {
    //"Constants"
    public static final NSI     ATTRNAME_NAME   = new NSI("name", null);
    public static final NSI     ATTRNAME_SIZE   = new NSI("size", null);
    public static final NSI     ATTRNAME_HASH   = new NSI("hash", null);
    public static final NSI     ATTRNAME_DATE   = new NSI("date", null);
    
    public static final NSI     ELEMNAME_DESC   = new NSI("desc", NAMESPACE);
    
    //Constructors
    public FTProfileNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected FTProfileNode(StreamElement parent, FTProfileNode base) {
        super(parent, base);
    }
    
    //Methods
    public StreamObject copy(StreamElement parent) {
        return new FTProfileNode(parent, this);
    }
    
    public String getName() {
        String      name = getAttributeValue(ATTRNAME_NAME);
        
        return ((name != null) ? name : "");
    }
    public void setName(String name) {
        setAttributeValue(ATTRNAME_NAME, Utilities.isValidString(name) ? name : null);
    }
    
    public long getSize() {
        Object      val = getAttributeObject(ATTRNAME_SIZE);
        long        size = 0;

        if          (val instanceof Number) {
            size = ((Number)val).longValue();
        } else if   (val != null) {
            try {
                size = Long.parseLong(val.toString());
                setSize(size);
            } catch (NumberFormatException nfe) {
            }
        }
        
        return size;
    }
    public void setSize(long size) {
        setAttributeObject(ATTRNAME_SIZE, (size >= 0) ? new Long(size) : null);
    }
    
    public byte[] getHash() {
        String      val = getAttributeValue(ATTRNAME_HASH);
        byte[]      hash;
        
        try {
            hash = ByteCodec.HEX.decode(val);
        } catch (IllegalArgumentException iae) {
            hash = new byte[0];
        }
        
        return hash;
    }
    public void setHash(byte[] hash) {
        setAttributeObject(ATTRNAME_HASH, (hash != null) ? ByteCodec.HEX.encode(hash) : null);
    }
    
    public Date getDate() {
        Object      val = getAttributeObject(ATTRNAME_DATE);
        Date        date = null;
        
        if          (val instanceof DateTimeProfileWrapper) {
            date = ((DateTimeProfileWrapper)val).toDate();
        } else if   (val != null) {
            try {
                val = DateTimeProfileWrapper.create(DateTimeProfileFormat.DATETIME, val.toString());
                date = ((DateTimeProfileWrapper)val).toDate();
                
                setAttributeObject(ATTRNAME_DATE, val);
            } catch (Exception e) {
            }
        }
        
        return date;
    }
    public void setDate(Date date) {
        setAttributeObject(ATTRNAME_DATE, (date != null) ? DateTimeProfileWrapper.create(DateTimeProfileFormat.DATETIME, date) : null);
    }
    
    public String getDescription() {
        StreamElement   elem = getFirstElement(ELEMNAME_DESC);
        
        return ((elem != null) ? elem.normalizeText() : "");
    }
    public void setDescription(String desc) {
        StreamElement   elem = getFirstElement(ELEMNAME_DESC);
        
        if          (Utilities.isValidString(desc)) {
            if (elem == null)
                elem = addElement(ELEMNAME_DESC);
            elem.addText(desc);
        } else if   (elem != null) {
            elem.detach();
        }
    }
    
    
    public boolean hasRange() {
        return (getFirstElement(FileTransferProfile.Range.class) != null);
    }
    public FileTransferProfile.Range getRange() {
        FileTransferProfile.Range   range = (FileTransferProfile.Range)getFirstElement(FileTransferProfile.Range.class);
        
        if (range == null) {
            range = (FileTransferProfile.Range)addElement(FileTransferProfile.Range.NAME, FileTransferProfile.Range.class);
        }
        
        return range;
    }
    public FileTransferProfile.Range putRange(long off, long len) throws IllegalArgumentException {
        FileTransferProfile.Range   range;
        
        if (off < 0)
            throw new IllegalArgumentException("offset cannot be less than 0");
        if (len < 0)
            throw new IllegalArgumentException("length cannot be less than 0");
        
        range = getRange();
        range.setOffset(off);
        range.setLength(len);
        
        return range;
    }
    public void removeRange() {
        clearElements(FileTransferProfile.Range.class);
    }
}