/*
 * AbstractFieldContainer.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.xdata;


import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.xdata.XDataField;
import org.jabberstudio.jso.x.xdata.XDataFieldContainer;
import org.jabberstudio.jso.x.xdata.XDataForm;

import net.outer_planes.jso.ElementNode;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public abstract class AbstractFieldContainer extends ElementNode implements XDataFieldContainer {
    //"Constants"
    public static final Map     NSS;
    
    static {
        NSS = Collections.unmodifiableMap(Collections.singletonMap("xdata", XDataForm.NAMESPACE));
    }
    
    //Constructors
    public AbstractFieldContainer(StreamDataFactory sdf, NSI name) {
        super(sdf, name);
    }
    public AbstractFieldContainer(StreamElement parent, AbstractFieldContainer cont) {
        super(parent, cont);
    }
    
    //Methods
    public List listFields() {
        return listElements(XDataField.class);
    }
    public XDataField getField(String var) throws IllegalArgumentException {
        XDataField  field = null;
        
        if (!Utilities.isValidString(var))
            throw new IllegalArgumentException("var cannot be null or \"\"");

        try {
            field = (XDataField)select("xdata:field[@var='" + var + "']", NSS);
        } catch (IllegalArgumentException iae) {
        }
        
        return field;
    }
    public XDataField addField(String var) throws IllegalArgumentException, IllegalStateException {
        return addField(var, null);
    }
    public XDataField addField(String var, XDataField.Type type) throws IllegalArgumentException {
        XDataField  field = getField(var);

        if          (field == null) {
            FieldNode   temp = (FieldNode)addElement(XDataField.NAME, FieldNode.class);
            
            temp.setType(type);
            temp.setVar(var);
            
            field = temp;
        } else if   (field.getType() != type) {
            throw new IllegalStateException("field with different type already exists for \"" + var + "\"");
        }
        
        return field;
    }
    public void removeField(String var) throws IllegalArgumentException {
        XDataField      field = getField(var);
        
        if (field != null)
            field.detach();
    }
    public void clearFields() {
        clearElements(XDataField.class);
    }

    public List listFieldValues(String var) throws IllegalArgumentException {
        XDataField  field = getField(var);
        
        return ((field != null) ? field.listValues() : Collections.EMPTY_LIST);
    }
    public String getFieldValue(String var) throws IllegalArgumentException {
        XDataField  field = getField(var);
        
        return ((field != null) ? field.getValue() : null);
    }
}