/*
 * FieldNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.xdata;

import java.util.Iterator;
import java.util.List;

import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.xdata.XDataField;
import org.jabberstudio.jso.x.xdata.XDataForm;

import net.outer_planes.jso.ElementNode;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class FieldNode extends ElementNode implements XDataField {
    //"Constants"
    public static final NSI     ELEMNAME_DESC       = new NSI("desc", XDataForm.NAMESPACE);
    public static final NSI     ELEMNAME_VALUE      = new NSI("value", XDataForm.NAMESPACE);
    public static final NSI     ELEMNAME_REQUIRED   = new NSI("required", XDataForm.NAMESPACE);
    
    public static final NSI     ATTRNAME_LABEL      = new NSI("label", null);
    public static final NSI     ATTRNAME_VAR        = new NSI("var", null);
    
    //Constructors
    public FieldNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected FieldNode(StreamElement parent, FieldNode base) {
        super(parent, base);
    }
    
    //Methods
    public XDataField.Type getType() {
        Object          val = getAttributeObject(FormNode.ATTRNAME_TYPE);
        String          name;
        XDataField.Type type = null;
        
        if      (val instanceof XDataField.Type)
            type = (XDataField.Type)val;
        else if (val != null) {
            name = val.toString();
            
            if          (Utilities.equateStrings(BOOLEAN.toString(), name)) {
                type = BOOLEAN;
            } else if   (Utilities.equateStrings(FIXED.toString(), name)) {
                type = FIXED;
            } else if   (Utilities.equateStrings(HIDDEN.toString(), name)) {
                type = HIDDEN;
            } else if   (Utilities.equateStrings(JID_MULTI.toString(), name)) {
                type = JID_MULTI;
            } else if   (Utilities.equateStrings(JID_SINGLE.toString(), name)) {
                type = JID_SINGLE;
            } else if   (Utilities.equateStrings(LIST_MULTI.toString(), name)) {
                type = LIST_MULTI;
            } else if   (Utilities.equateStrings(LIST_SINGLE.toString(), name)) {
                type = LIST_SINGLE;
            } else if   (Utilities.equateStrings(TEXT_MULTI.toString(), name)) {
                type = TEXT_MULTI;
            } else if   (Utilities.equateStrings(TEXT_PRIVATE.toString(), name)) {
                type = TEXT_PRIVATE;
            } else if   (Utilities.equateStrings(TEXT_SINGLE.toString(), name)) {
                type = TEXT_SINGLE;
            }

            if (type != null)
                setAttributeObject(FormNode.ATTRNAME_TYPE, type);
        }
        
        return type;
    }
    public void setType(XDataField.Type type) {
        if (type == null)
            type = TEXT_SINGLE;
        setAttributeObject(FormNode.ATTRNAME_TYPE, type);
    }
    
    public String getVar() {
        String  val = getAttributeValue(ATTRNAME_VAR);
        
        return ((val != null) ? val : "");
    }
    public void setVar(String var) {
        setAttributeValue(ATTRNAME_VAR, var);
    }
    
    public boolean hasLabel() {
        return Utilities.isValidString(getAttributeValue(ATTRNAME_LABEL));
    }
    public String getLabel() {
        String  val = getAttributeValue(ATTRNAME_LABEL);
        
        return (Utilities.isValidString(val) ? val : getVar());
    }
    public void setLabel(String label) {
        setAttributeValue(ATTRNAME_LABEL, Utilities.isValidString(label) ? label : null);
    }
    
    public boolean isRequired() {
        return (getFirstElement(ELEMNAME_REQUIRED) != null);
    }
    public void setRequired(boolean required) {
        StreamElement   elem = getFirstElement(ELEMNAME_REQUIRED);
        
        if      (required && (elem == null))
            elem = addElement(ELEMNAME_REQUIRED);
        else if (!required && (elem != null))
            elem.detach();
    }
    
    public String getDescription() {
        StreamElement   elem = getFirstElement(ELEMNAME_DESC);
        
        return ((elem != null) ? elem.normalizeTrimText() : "");
    }
    public void setDescription(String desc) {
        StreamElement   elem = getFirstElement(ELEMNAME_DESC);
        
        if          (Utilities.isValidString(desc)) {
            if (elem != null) {
                elem.clearText();
            } else {
                elem = addElement(ELEMNAME_DESC);
            }
            elem.addText(desc);
        } else if   (elem != null) {
            elem.detach();
        }
    }
    
    public List listOptions() {
        return listElements(XDataField.Option.class);
    }
    public XDataField.Option getOption(String value) {
        Iterator            itr = listOptions().iterator();
        XDataField.Option   opt = null;

        if (value == null)
            value = "";
        
        while ((opt == null) && itr.hasNext()) {
            opt = (XDataField.Option)itr.next();
            
            if (!Utilities.equateStrings(opt.getValue(), value))
                opt = null;
        }
        
        return opt;
    }
    public XDataField.Option addOption(String value) {
        return addOption(null, value);
    }
    public XDataField.Option addOption(String label, String value) {
        XDataField.Option   opt;
        
        if (value == null)
            value = "";
        
        opt = getOption(value);
        if (opt == null) {
            OptionNode  temp = (OptionNode)addElement(OptionNode.NAME, OptionNode.class);
            
            temp.setLabel(label);
            temp.setValue(value);
            
            opt = temp;
        }
        
        return opt;
    }
    public void removeOption(String value) {
        Iterator        opts = listOptions().iterator();
        
        if (value == null)
            value = "";
        
        while (opts.hasNext()) {
            XDataField.Option   opt = (XDataField.Option)opts.next();
            
            if (Utilities.equateStrings(value, opt.getValue()))
                opt.detach();
        }
    }
    public void clearOptions() {
        clearElements(XDataField.Option.class);
    }

    public List listValues() {
        List        elems = listElements(ELEMNAME_VALUE);
        List        vals = new java.util.ArrayList(elems.size());
        Iterator    itr = elems.iterator();
        
        while (itr.hasNext()) {
            StreamElement   elem = (StreamElement)itr.next();
            
            vals.add(elem.normalizeTrimText());
        }
        
        return vals;
    }
    public String getValue() {
        StreamElement   elem = getFirstElement(ELEMNAME_VALUE);
        
        return ((elem != null) ? elem.normalizeTrimText() : null);
    }
    public void setValue(String val) {
        clearValues();
        if (val != null)
            addValue(val);
    }
    public void addValue(String val) {
        addElement(ELEMNAME_VALUE).addText((val != null) ? val: "");
    }
    public void removeValue(String val) {
        Iterator        itr = listElements(ELEMNAME_VALUE).iterator();
        boolean         found = false;
        
        if (val == null)
            val = "";

        while (!found && itr.hasNext()) {
            StreamElement   elem = (StreamElement)itr.next();
            
            if (Utilities.equateStrings(elem.normalizeTrimText(), val)) {
                elem.detach();
                found = true;
            }
        }
    }
    public void clearValues() {
        clearElements(ELEMNAME_VALUE);
    }
    
    public Boolean getBooleanValue() throws IllegalStateException {
        String          temp = getValue();
        XDataField.Type type = getType();
        Boolean         val = null;
        
        if (temp != null) {
            val = (temp.equals("1") ? Boolean.TRUE : Boolean.FALSE);
        }
        
        return val;
    }
    public void setBooleanValue(Boolean val) {
        setValue((val != null) ? (val.booleanValue() ? "1" : "0") : null);
    }

    public JID getJIDValue() throws IllegalStateException {
        XDataField.Type type = getType();
        String          temp = getValue();
        JID             val = null;
        
        if (Utilities.isValidString(temp)) {
            try {
                val = JID.valueOf(temp);
            } catch (IllegalArgumentException iae) {
            }
        }
        
        return val;
    }
    public void setJIDValue(JID val) throws IllegalStateException {
        XDataField.Type type = getType();
        
        setValue((val != null) ? val.toString() : "");
    }
    public List listJIDValues() {
        XDataField.Type type = getType();
        List            vals = listValues();
        List            jids = new java.util.ArrayList(vals.size());
        Iterator        itr = vals.iterator();
        
        while (itr.hasNext()) {
            String  temp = (String)itr.next();
            JID     jid = null;
            
            try {
                jids.add(JID.valueOf(temp));
            } catch (IllegalArgumentException iae) {
            }
        }
        
        return jids;
    }
    public void addJIDValue(JID val) {
        XDataField.Type type = getType();
        
        addValue((val != null) ? val.toString() : "");
    }
    public void removeJIDValue(JID val) {
        XDataField.Type type = getType();
        
        removeValue((val != null) ? val.toString() : "");
    }
    
    public StreamObject copy(StreamElement parent) {
        return new FieldNode(parent, this);
    }
}