/*
 * FormNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.xdata;


import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.xdata.XDataField;
import org.jabberstudio.jso.x.xdata.XDataForm;
import org.jabberstudio.jso.x.xdata.XDataItem;
import org.jabberstudio.jso.x.xdata.XDataReported;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class FormNode extends ExtensionNode implements XDataForm {
    //"Constants"
    public static final NSI     ELEMNAME_TITLE          = new NSI("title", NAMESPACE);
    public static final NSI     ELEMNAME_INSTRUCTIONS   = new NSI("instructions", NAMESPACE);
    
    public static final NSI     ATTRNAME_TYPE           = new NSI("type", null);
    
    //Constructors
    public FormNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected FormNode(StreamElement parent, FormNode base) {
        super(parent, base);
    }

    //Methods
    public XDataForm.Type getType() {
        Object          val = getAttributeObject(ATTRNAME_TYPE);
        String          name;
        XDataForm.Type  type = null;
        
        if      (val instanceof XDataForm.Type)
            type = (XDataForm.Type)val;
        else if (val != null) {
            name = val.toString();
            
            if          (Utilities.equateStrings(CANCEL.toString(), name)) {
                type = CANCEL;
            } else if   (Utilities.equateStrings(FORM.toString(), name)) {
                type = FORM;
            } else if   (Utilities.equateStrings(RESULT.toString(), name)) {
                type = RESULT;
            } else if   (Utilities.equateStrings(SUBMIT.toString(), name)) {
                type = SUBMIT;
            }
            
            if (type != null)
                setAttributeObject(ATTRNAME_TYPE, type);
        }
        
        return type;
    }
    public void setType(XDataForm.Type type) {
        setAttributeObject(ATTRNAME_TYPE, type);
    }
    
    public String getTitle() {
        StreamElement       elem = getFirstElement(ELEMNAME_TITLE);
        
        return ((elem != null) ? elem.normalizeTrimText() : "");
    }
    public void setTitle(String title) {
        StreamElement       elem = getFirstElement(ELEMNAME_TITLE);
        
        if          (Utilities.isValidString(title)) {
            if (elem != null) {
                elem.clearText();
            } else {
                elem = addElement(ELEMNAME_TITLE);
            }
            elem.addText(title);
        } else if   (elem != null) {
            elem.detach();
        }
    }
    
    public List listInstructions() {
        List        elems = listElements(ELEMNAME_INSTRUCTIONS);
        List        lines = new java.util.ArrayList();
        Iterator    itr = elems.iterator();
        
        while (itr.hasNext())
            lines.add(((StreamElement)itr.next()).normalizeTrimText());
        
        return lines;
    }
    public void addInstruction(String inst) {
        StreamElement       elem = addElement(ELEMNAME_INSTRUCTIONS);
        
        if (Utilities.isValidString(inst))
            elem.addText(inst);
    }
    public void removeInstruction(String inst) {
        Iterator        itr = listElements(ELEMNAME_INSTRUCTIONS).iterator();
        StreamElement   elem;
        String          test;
        boolean         removed = false;
        
        while (!removed && itr.hasNext()) {
            elem = (StreamElement)itr.next();
            test = elem.normalizeTrimText();
            
            if (Utilities.equateStrings(test, inst)) {
                elem.detach();
                removed = true;
            }
        }
    }
    public void clearInstructions() {
        clearElements(ELEMNAME_INSTRUCTIONS);
    }
    
    public String getInstructions() {
        Iterator        itr = listInstructions().iterator();
        StreamElement   elem;
        StringBuffer    buffer = new StringBuffer();
        String          sep = System.getProperty("line.separator");

        while (itr.hasNext()) {
            if (buffer.length() > 0)
                buffer.append(sep);
            
            buffer.append(itr.next());
        }
        
        return buffer.toString();
    }
    public void setInstructions(String inst) {
        Iterator            itr;
        String              line;
        StreamElement       elem;
        
        clearInstructions();
        if (Utilities.isValidString(inst)) {
            itr = Arrays.asList(inst.split(System.getProperty("line.separator"), -1)).iterator();
            
            while (itr.hasNext()) {
                line = ((String)itr.next()).trim();
                elem = addElement(ELEMNAME_INSTRUCTIONS);
                
                if (Utilities.isValidString(line))
                    elem.addText(line);
            }
        }
    }

    public XDataReported getReported() {
        return (XDataReported)getFirstElement(XDataReported.class);
    }
    public XDataReported addReported() {
        XDataReported   report = getReported();
        
        if (report == null) {
            report = (XDataReported)addElement(XDataReported.NAME, XDataReported.class);
        }
        
        return report;
    }
    public void removeReported() {
        clearElements(XDataReported.class);
    }
    
    public List listItems() {
        return listElements(XDataItem.class);
    }
    public XDataItem addItem() {
        return (XDataItem)addElement(XDataItem.NAME, XDataItem.class);
    }
    public void clearItems() {
        clearElements(XDataItem.class);
    }
    
    public List listFields() {
        return listElements(XDataField.class);
    }
    public XDataField getField(String var) throws IllegalArgumentException {
        XDataField  field = null;
        
        if (!Utilities.isValidString(var))
            throw new IllegalArgumentException("var cannot be null or \"\"");

        try {
            field = (XDataField)select("xdata:field[(@var='" + var + "') and implements-class('org.jabberstudio.jso.x.xdata.XDataField')]", AbstractFieldContainer.NSS);
        } catch (IllegalArgumentException iae) {
        }
        
        return field;
    }
    public XDataField addField(String var) throws IllegalArgumentException, IllegalStateException {
        return addField(var, null);
    }
    public XDataField addField(String var, XDataField.Type type) throws IllegalArgumentException {
        XDataField  field = getField(var);
        
        if (type == null)
            type = XDataField.TEXT_SINGLE;
        
        if          (field == null) {
            FieldNode   temp = (FieldNode)addElement(XDataField.NAME, FieldNode.class);
            
            temp.setType(type);
            temp.setVar(var);
            
            field = temp;
        } else if   (field.getType() != type) {
            throw new IllegalStateException("field with different type already exists for \"" + var + "\"");
        }
        
        return field;
    }
    public void removeField(String var) throws IllegalArgumentException {
        XDataField      field = getField(var);
        
        if (field != null)
            field.detach();
    }
    public void clearFields() {
        clearElements(XDataField.class);
    }

    public List listFieldValues(String var) throws IllegalArgumentException {
        XDataField  field = getField(var);
        
        return ((field != null) ? field.listValues() : Collections.EMPTY_LIST);
    }
    public String getFieldValue(String var) throws IllegalArgumentException {
        XDataField  field = getField(var);
        
        return ((field != null) ? field.getValue() : null);
    }
    
    public XDataField addFixedField(String val) {
        XDataField      field = (XDataField)addElement(XDataField.NAME, XDataField.class);
    
        field.setType(XDataField.FIXED);
        field.setValue((val == null) ? "" : val);
        
        return field;
    }
    
    public StreamObject copy(StreamElement parent) {
        return new FormNode(parent, this);
    }
}