/*
 * Extension.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


/**
 * <p>
 * Interface used to define the "extension" data mechanism. This interface
 * provides the minimum information that "extended data" must provide.</p>
 *
 * <p>
 * Extensions in JABBER are usually represented by the &lt;x/&gt; element (for
 * "presence" and "message"), or &lt;query&gt; element (for "iq").</p>
 *
 * <p>
 * Of important note is that the following methods are considered "unsupported"
 * for most instances of this interface:
 * <ul>
 * <li><code>{@link #insert}</code></li>
 * </ul></p>
 *
 * @author linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since JSO-0.2
 */
public interface Extension extends StreamElement {
    //Methods
    /**
     * <p>
     * Retrieves the namespace URI this <code>Extension</code> is
     * defined by.  The namespace is used to distiguish the type
     * of the <code>Extension</code>.</p>
     *
     * <p>
     * This method never returns <code>null</code> or "".</p>
     *
     * @return The namespace URI.
     * @since   JSO-0.2
     */    
    public String getNamespaceURI();
}