/*
 * SASLFailurePacket.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl;


import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Interface specifically for representing SASL failure packets.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface SASLFailurePacket extends SASLPacket {
    //"Constants"
    /** Qualified name for <tt>SASLFailurePacket</tt>
     * ({urn:ietf:params:xml:ns:xmpp-sasl}failure).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME                                = NAME_FAILURE;

    /** Failure condition for aborted authentication attempt.
     *
     * @since   JSO-0.12
     */
    public static final String  ABORTED_CONDITION                   = "aborted";
    /** Failure condition for incorrect data or parameter encoding.
     *
     * @since   JSO-0.12
     */
    public static final String  INCORRECT_ENCODING_CONDITION        = "incorrect-encoding";
    /** Failure condition for invalid authorization identifier specified.
     *
     * @since   JSO-0.12
     */
    public static final String  INVALID_AUTHZID_CONDITION           = "invalid-authzid";
    /** Failure condition for invalid mechanism specified.
     *
     * @since   JSO-0.12
     */
    public static final String  INVALID_MECHANISM_CONDITION         = "invalid-mechanism";
    /** Failure condition for using too weak a mechanism.
     *
     * @since   JSO-0.12
     */
    public static final String  MECHANISM_TOO_WEAK_CONDITION        = "mechanism-too-weak";
    /** Failure condition for specifying authorization is denied.
     *
     * @since   JSO-0.12
     */
    public static final String  NOT_AUTHORIZED_CONDITION            = "not-authorized";
    /** Failure condition for a temporary authentication failure.
     *
     * @since   JSO-0.12
     */
    public static final String  TEMPORARY_AUTH_FAILURE_CONDITION    = "temporary-auth-failure";
    
    //Methods
    /**
     * <p>
     * Retrieves the failure condition for this <tt>SASLFailurePacket</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The failure condition.
     * @since   JSO-0.12
     */
    public String getCondition();
    /**
     * <p>
     * Sets the failure condition for this <tt>SASLFailurePacket</tt>.</p>
     *
     * <p>
     * The value of <tt>cond</tt> may be <tt>null</tt> or "", in which case
     * any previous value is removed. If the value of <tt>cond</tt> does not
     * match a known condition, an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param cond The new failure condition.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public void setCondition(String cond) throws IllegalArgumentException;
}