/*
 * NameprepProfile.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */


package org.jabberstudio.jso.util.stringprep;

/**
 * <p>
 * Implements the nameprep profile, as defined in RFC 3491.</p>
 *
 * @see     <a href='http://www.ietf.org/rfc/rfc3491.txt'>Nameprep: A Stringprep Profile for Internationalized Domain Names (IDN)</a>
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12.3
 */
public class NameprepProfile extends StringprepProfile {
    //Fields
    private static  NameprepProfile     _Inst;

    //Constructors
    /**
     * <p>
     * Creates a new <tt>NameprepProfile</tt>.</p>
     *
     * @since   JSO-0.12.3
     */
    protected NameprepProfile() {
        super(  true,
                true,
                new MappingTable[] {
                    MAPPING_B_1,
                    MAPPING_B_2
                },
                new ProhibitedTable[] {
                    PROHIBITED_C_1_2,
                    PROHIBITED_C_2_2,
                    PROHIBITED_C_3,
                    PROHIBITED_C_4,
                    PROHIBITED_C_5,
                    PROHIBITED_C_6,
                    PROHIBITED_C_7,
                    PROHIBITED_C_8,
                });
    }

    //Methods
    /**
     * <p>
     * Retrieves the global instance of <tt>NameprepProfile</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return  The global <tt>NameprepProfile</tt> instance.
     * @since   JSO-0.12.3
     */
    public static synchronized NameprepProfile getInstance() {
        if (_Inst == null)
            _Inst = new NameprepProfile();
        return _Inst;
    }
}