/*
 * LastQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.info;


import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Interface for representing "jabber:iq:last" queries, as documented in
 * JEP-0012.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 * @see <a href="http://www.jabber.org/jeps/jep-0012.html">JEP-0012: Last Activity</a>
 */
public interface LastQuery extends Extension {
    //"Constants"
    /**
     * The namespace URI for <tt>LastQuery</tt> (jabber:iq:last).
     *
     * @since   JSO-0.12
     */
    public static final String      NAMESPACE   = "jabber:iq:last";
    /**
     * The qualified name for <tt>LastQuery</tt> ({jabber:iq:last}query).
     *
     * @since   JSO-0.12
     */
    public static final NSI         NAME        = new NSI("query", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the seconds for this <tt>LastQuery</tt>. This value is
     * the number of seconds the application has been online (if for a server
     * or component), the number of seconds of idle activity (if for an online
     * user), or the number of seconds since the last login (if for an offline
     * user).</p>
     *
     * <p>
     * The value returned by this method should never be less than or equal
     * to 0.</p>
     *
     * @return The number of seconds.
     * @since   JSO-0.12
     */
    public long getSeconds();
    /**
     * <p>
     * Sets the number of seconds for this <tt>LastQuery</tt>.
     *
     * <p>
     * If the value of <tt>secs</tt> may be less than 0, in which case it is
     * considered to be 0.</p>
     *
     * @param secs The new number of seconds.
     * @since   JSO-0.12
     */
    public void setSeconds(long secs);
}